import { Component, ElementRef, Input, ViewChild } from '@angular/core';
import { AbstractControlDirective, NgControl } from '@angular/forms';
import { Observable } from 'rxjs/Observable';
var RlDatePickerInputComponent = (function () {
    function RlDatePickerInputComponent(el, control) {
        var _this = this;
        this.el = el;
        this.control = control;
        this.options = {
            format: 'MM/dd/yyyy',
            minDate: null,
            maxDate: null,
            showWeeks: false
        };
        this.opened = false;
        this.dateObject = new Date();
        this.ngOnDestroy = function () { return _this.removeEventListener(); };
    }
    RlDatePickerInputComponent.prototype.ngOnInit = function () {
        this.dateObject = this.ngModel || this.control.value;
    };
    RlDatePickerInputComponent.prototype.ngDoCheck = function () {
        if (this.dateObject !== this.$dateObject) {
            this.$dateObject = this.dateObject;
            this.control.control.setValue(this.dateObject);
        }
    };
    Object.defineProperty(RlDatePickerInputComponent.prototype, "LayoutElement", {
        /**
         * Get the HTMLElement of this Element
         * @returns {HTMLElement}
         * @constructor
         */
        get: function () {
            return this.el.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RlDatePickerInputComponent.prototype, "DatepickerElement", {
        /**
         * Get the HTMLElement of the Datepicker
         * @returns {HTMLElement}
         * @constructor
         */
        get: function () {
            return this.LayoutElement.children[1];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RlDatePickerInputComponent.prototype, "OpenBtnElement", {
        get: function () {
            return (this.OpenBtn.nativeElement || this.OpenBtn);
        },
        enumerable: true,
        configurable: true
    });
    RlDatePickerInputComponent.prototype.removeEventListener = function () {
        if (this.subscriber)
            this.subscriber.unsubscribe();
    };
    RlDatePickerInputComponent.prototype.addEventListener = function () {
        var _this = this;
        this.docmousedown = Observable.fromEvent(document, 'mousedown');
        this.subscriber = this.docmousedown.subscribe(function (e) {
            e.stopPropagation();
            e.preventDefault();
            var btnClicked = _this.OpenBtnElement.contains(e.target) === false;
            var dtClicked = _this.DatepickerElement.contains(e.target) === false;
            if (btnClicked && dtClicked)
                _this.opened = false;
        });
    };
    RlDatePickerInputComponent.prototype.open = function () {
        this.opened = !this.opened;
        this.opened ? this.addEventListener() : this.removeEventListener();
    };
    RlDatePickerInputComponent.prototype.setToToday = function () {
        this.dateObject = new Date();
        this.opened = false;
    };
    RlDatePickerInputComponent.prototype.clearDate = function () {
        this.dateObject = undefined;
        this.opened = false;
    };
    __decorate([
        ViewChild('OpenBtn'),
        __metadata("design:type", ElementRef)
    ], RlDatePickerInputComponent.prototype, "OpenBtn", void 0);
    __decorate([
        Input('ngModel'),
        __metadata("design:type", AbstractControlDirective)
    ], RlDatePickerInputComponent.prototype, "ngModel", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], RlDatePickerInputComponent.prototype, "options", void 0);
    RlDatePickerInputComponent = __decorate([
        Component({
            selector: 'rl-datepicker-input',
            templateUrl: './datepicker-input.component.html'
        }),
        __metadata("design:paramtypes", [ElementRef,
            NgControl])
    ], RlDatePickerInputComponent);
    return RlDatePickerInputComponent;
}());
export { RlDatePickerInputComponent };
//# sourceMappingURL=datepicker-input.component.js.map