import {
    Component,
    DoCheck,
    ElementRef,
    Input,
    OnDestroy,
    OnInit,
    Query,
    ViewChild,
    ViewContainerRef
} from '@angular/core';
import { AbstractControlDirective, NgControl } from '@angular/forms';
import { Observable } from 'rxjs/Observable';

@Component ({
  selector   : 'rl-datepicker-input',
  templateUrl: './datepicker-input.component.html'
})

export class RlDatePickerInputComponent implements OnInit, DoCheck, OnDestroy {

  @ViewChild('OpenBtn') protected OpenBtn: ElementRef;
  @Input ('ngModel') ngModel: AbstractControlDirective;

  @Input () options: any = {
    format   : 'MM/dd/yyyy',
    minDate  : null,
    maxDate  : null,
    showWeeks: false
  };


  protected opened: boolean = false;
  protected dateObject = new Date ();

  //keep a copy to compare
  protected $dateObject;
  protected docmousedown;
  protected subscriber;


  constructor (protected el: ElementRef,
               protected control: NgControl) {
  }

  ngOnInit (): void {
    this.dateObject = this.ngModel || this.control.value;
    
  }

  ngOnDestroy = (): void => this.removeEventListener ();


  ngDoCheck (): void {
    if (this.dateObject !== this.$dateObject) {
      this.$dateObject = this.dateObject;
      this.control.control.setValue (this.dateObject);
    }
  }

  /**
   * Get the HTMLElement of this Element
   * @returns {HTMLElement}
   * @constructor
   */
  protected get LayoutElement (): HTMLElement {
    return <HTMLElement>(<Element>this.el.nativeElement);
  }

  /**
   * Get the HTMLElement of the Datepicker
   * @returns {HTMLElement}
   * @constructor
   */
  protected get DatepickerElement (): HTMLElement {
    return <HTMLElement>(<Element>this.LayoutElement.children[ 1 ]);
  }

  protected get OpenBtnElement () {
    return <HTMLElement>(<Element>this.OpenBtn.nativeElement || this.OpenBtn);
  }

  protected removeEventListener () {
    if(this.subscriber)
      this.subscriber.unsubscribe ();
  }

  protected addEventListener () {
    this.docmousedown = Observable.fromEvent (document, 'mousedown');

    this.subscriber = this.docmousedown.subscribe ((e) => {
      e.stopPropagation ();
      e.preventDefault ();
      
      const btnClicked = this.OpenBtnElement.contains(e.target) === false;
      const dtClicked = this.DatepickerElement.contains (e.target) === false;

      if (btnClicked && dtClicked)
        this.opened = false;

    });
  }

  open () {
    this.opened = !this.opened;
    this.opened ? this.addEventListener () : this.removeEventListener ();
  }

  setToToday () {
    this.dateObject = new Date ();
    this.opened = false;
  }

  clearDate () {
    this.dateObject = undefined;
    this.opened = false;
  }
}


